import junit.framework.TestCase;

import org.infinispan.Cache;
import org.infinispan.manager.CacheManager;
import org.infinispan.manager.DefaultCacheManager;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Properties;
import java.util.WeakHashMap;
import java.util.concurrent.ConcurrentHashMap;

public class TestNullValue extends TestCase
{
   private CacheManager manager;

   private Cache<String, String> cache;

   @Override
   protected void setUp() throws Exception
   {
      manager = new DefaultCacheManager();
      cache = manager.getCache();
   }

   @Override
   protected void tearDown() throws Exception
   {
      cache.stop();
      manager.stop();
   }

   @SuppressWarnings("unchecked")
   public void testNullValue() throws Exception
   {
      Map[] maps =
         {new HashMap<String, String>(), new ConcurrentHashMap<String, String>(), new LinkedHashMap<String, String>(),
            new Properties(), new WeakHashMap<String, String>(), cache};
      for (Map<String, String> map : maps)
      {
         testNullValue(map);
      }
   }

   private void testNullValue(Map<String, String> cache) throws Exception
   {
      System.out.println("Test Null Value with " + cache.getClass());
      boolean supported = false;
      try
      {
         cache.put("a", null);
         // Null value supported
         supported = true;
      }
      catch (NullPointerException e)
      {
         // Null Value not supported : OK
      }
      catch (Exception e)
      {
         fail("Only NullPointerException is allowed");
      }
      System.out.println("The map of type " + cache.getClass() + " " + (supported ? "supports" : "doesn't support")
         + " null values");
      cache.put("a", "b");
      try
      {
         cache.put("a", null);
         // Null value supported
         if (supported)
         {
            assertNull("As " + cache.getClass()+ " supports null value we expect 'null' as value", cache.get("a"));
         }
         else
         {
            fail("'null values' are not supposed to be supported by " + cache.getClass());
         }
      }
      catch (NullPointerException e)
      {
         // Null Value not supported : OK
         if (supported)
         {
            fail("'null values' are supposed to be supported by " + cache.getClass());
         }
         else
         {
            assertEquals("As " + cache.getClass()+ " doesn't support null value we expect 'b' as value", "b", cache.get("a"));
         }
      }
      catch (Exception e)
      {
         fail("Only NullPointerException is allowed");
      }
   }
}
